@@ -32,6 +32,13 @@
 #include "gmime-content-type.h"
 #include "gmime-table-private.h"
 
+
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
 #define d(x)
 
 
@@ -90,9 +97,9 @@ g_mime_content_type_new (const char *type, const char *subtype)
 			mime_type->subtype = g_strdup ("octet-stream");
 		}
 		
-		g_warning ("Invalid or incomplete type: %s%s%s: defaulting to %s/%s",
-			   type ? type : "", subtype ? "/" : "", subtype ? subtype : "",
-			   mime_type->type, mime_type->subtype);
+		w(g_warning ("Invalid or incomplete type: %s%s%s: defaulting to %s/%s",
+			     type ? type : "", subtype ? "/" : "", subtype ? subtype : "",
+			     mime_type->type, mime_type->subtype));
 	}
 	
 	return mime_type;
@@ -129,12 +136,15 @@ g_mime_content_type_new_from_string (const char *string)
 	/* get the subtype */
 	if (*inptr == '/') {
 		inptr++;
+		
+		while (is_lwsp (*inptr))
+			inptr++;
+		
 		subtype = (char *) inptr;
 		while (*inptr && is_ttoken (*inptr))
 			inptr++;
 		
 		subtype = g_strndup (subtype, (unsigned) (inptr - subtype));
-		g_strstrip (subtype);
 	} else {
 		subtype = NULL;
 	}
@@ -143,6 +153,9 @@ g_mime_content_type_new_from_string (const char *string)
 	g_free (type);
 	g_free (subtype);
 	
+	while (is_lwsp (*inptr))
+		inptr++;
+	
 	if (*inptr++ == ';' && *inptr) {
 		GMimeParam *p;
 		
@@ -231,7 +244,7 @@ g_mime_content_type_is_type (const GMimeContentType *mime_type, const char *type
 	g_return_val_if_fail (type != NULL, FALSE);
 	g_return_val_if_fail (subtype != NULL, FALSE);
 	
-	if (!strcasecmp (mime_type->type, type)) {
+	if (!strcmp (type, "*") || !strcasecmp (mime_type->type, type)) {
 		if (!strcmp (subtype, "*")) {
 			/* special case */
 			return TRUE;
