@@ -858,8 +858,8 @@ parser_step_headers (GMimeParser *parser)
 			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
-				priv->inptr = start;
-				len = inend - start;
+				priv->inptr = inptr;
+				len = inend - inptr;
 				goto refill;
 			}
 			
@@ -868,7 +868,7 @@ parser_step_headers (GMimeParser *parser)
 				goto headers_end;
 			
 			len = inptr - start;
-			if (inptr[-1] == '\r')
+			if (inptr > start && inptr[-1] == '\r')
 				len--;
 			
 			header_append (priv, start, len);
