@@ -976,6 +976,35 @@ g_mime_references_clear (GMimeReferences **refs)
 }
 
 
+static gboolean
+is_rfc2047_token (const char *inptr, size_t len)
+{
+	if (len < 8 || strncmp (inptr, "=?", 2) != 0 || strncmp (inptr + len - 2, "?=", 2) != 0)
+		return FALSE;
+	
+	inptr += 3;
+	len -= 3;
+	
+	while (*inptr != '?' && len > 0) {
+		inptr++;
+		len--;
+	}
+	
+	if (*inptr != '?' || len < 4)
+		return FALSE;
+	
+	if (inptr[1] != 'q' && inptr[1] != 'Q' && inptr[1] != 'b' && inptr[1] != 'B')
+		return FALSE;
+	
+	inptr += 2;
+	len -= 2;
+	
+	if (*inptr != '?')
+		return FALSE;
+	
+	return TRUE;
+}
+
 static char *
 header_fold (const char *in, gboolean structured)
 {
@@ -1012,7 +1041,7 @@ header_fold (const char *in, gboolean structured)
 				outlen = 1;
 			}
 			
-			if (!structured) {
+			if (!structured && !is_rfc2047_token (inptr, len)) {
 				/* check for very long words, just cut them up */
 				while (outlen + len > GMIME_FOLD_LEN) {
 					for (i = 0; i < GMIME_FOLD_LEN - outlen; i++)
@@ -1041,7 +1070,8 @@ header_fold (const char *in, gboolean structured)
 				   mailer folded it */
 				g_string_append (out, "\n\t");
 				outlen = 1;
-				inptr++;
+				while (isblank (*inptr))
+					inptr++;
 			} else {
 				g_string_append_c (out, *inptr++);
 				outlen++;
@@ -2059,7 +2089,7 @@ g_mime_utils_header_encode_text (const unsigned char *in)
  * Decodes an rfc2047 encoded header.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_decode_text() instead.
+ * #g_mime_utils_header_decode_text instead.
  *
  * Returns the decoded header (which will be in UTF-8 if at all
  * possible).
@@ -2078,7 +2108,7 @@ g_mime_utils_8bit_header_decode (const unsigned char *in)
  * Encodes a 'text' header according to the rules in rfc2047.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_encode_text() instead.
+ * #g_mime_utils_header_encode_text instead.
  *
  * Returns the encoded header. Useful for encoding
  * headers like "Subject".
@@ -2097,7 +2127,7 @@ g_mime_utils_8bit_header_encode (const unsigned char *in)
  * Encodes a 'phrase' header according to the rules in rfc2047.
  *
  * WARNING: This function is deprecated. Use
- * g_mime_utils_header_encode_phrase() instead.
+ * #g_mime_utils_header_encode_phrase instead.
  *
  * Returns the encoded 'phrase'. Useful for encoding internet
  * addresses.
@@ -2118,7 +2148,7 @@ g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
  * @save: leftover bits that have not yet been encoded
  *
  * Base64 encodes the input stream to the output stream. Call this
- * when finished encoding data with g_mime_utils_base64_encode_step()
+ * when finished encoding data with #g_mime_utils_base64_encode_step
  * to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
@@ -2319,7 +2349,7 @@ g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned
  * @save: leftover bits that have not yet been encoded
  *
  * Uuencodes a chunk of data. Call this when finished encoding data
- * with g_mime_utils_uuencode_step() to flush off the last little bit.
+ * with #g_mime_utils_uuencode_step to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
@@ -2580,7 +2610,7 @@ g_mime_utils_uudecode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been encoded
  *
  * Quoted-printable encodes a block of text. Call this when finished
- * encoding data with g_mime_utils_quoted_encode_step() to flush off
+ * encoding data with #g_mime_utils_quoted_encode_step to flush off
  * the last little bit.
  *
  * Returns the number of bytes encoded.
@@ -2712,7 +2742,7 @@ g_mime_utils_quoted_encode_step (const unsigned char *in, size_t inlen, unsigned
 
 
 /**
- * g_mime_utils_quoted_decode_step: decode a chunk of QP encoded data
+ * g_mime_utils_quoted_decode_step:
  * @in: input stream
  * @inlen: max length of data to decode
  * @out: output stream
